@@ -29,7 +29,6 @@ def inputData(self, ScriptType: str = None):
 
     }
 
-
     CS_CA_CM_data_dict = {
         0: {
             'name': '0 = ESP CTE',
@@ -426,11 +425,17 @@ def inputData(self, ScriptType: str = None):
                 self.HVACmode_List = list(
                     int(num) for num in input("     Enter the HVACmode numbers separated by space: ").split())
 
-        fullVentCtrlList = [0, 1]
+        fullVentCtrlList = [0, 1, 2, 3]
         self.VentCtrl_List = list(int(num) for num in input(
             "Enter the Ventilation Control numbers separated by space (\n"
-            "0 = Ventilates above neutral temperature;\n"
-            "1 = Ventilates above upper comfort limit;\n"
+            "If HVACmode = 1:\n"
+            "   0 = Ventilates above neutral temperature;\n"
+            "   1 = Ventilates above upper comfort limit;\n"
+            "If HVACmode = 2:\n"
+            "   0 = Ventilates above neutral temperature and fully opens doors and windows;\n"
+            "   1 = Ventilates above lower comfort limit and fully opens doors and windows;\n"
+            "   2 = Ventilates above neutral temperature and opens doors and windows based on the customised venting opening factor;\n"
+            "   3 = Ventilates above lower comfort limit and opens doors and windows based on the customised venting opening factor;\n"
             "): ").split())
         while len(self.VentCtrl_List) == 0 or not all(elem in fullVentCtrlList for elem in self.VentCtrl_List):
             print('          Ventilation Control numbers are not correct. Please enter the numbers again.')
@@ -444,6 +449,59 @@ def inputData(self, ScriptType: str = None):
                 self.VentCtrl_List = list(int(num) for num in input(
                     "     Enter the Ventilation Control numbers separated by space: ").split())
 
+        if any([i in self.VentCtrl_List for i in [2, 3]]):
+            self.MaxTempDiffVOF = float(input('Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
+            while self.MaxTempDiffVOF <= 0:
+                print('          The maximum temperature difference number is not correct. It must be a number larger than 0. Please enter the number again.')
+                self.MaxTempDiffVOF = float(input('         Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
+            while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
+                self.MaxTempDiffVOF = float(input('      Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
+                while self.MaxTempDiffVOF <= 0:
+                    print('          The maximum temperature difference number is not correct. It must be a number larger than 0. Please enter the number again.')
+                    self.MaxTempDiffVOF = float(input('         Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
+
+            self.MinTempDiffVOF = float(input('Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
+            while self.MinTempDiffVOF <= 0:
+                print('          The minimum temperature difference number is not correct. It must be a number larger than 0 and smaller than the maximum temperature difference number. Please enter the number again.')
+                self.MinTempDiffVOF = float(input('         Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
+            while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
+                self.MinTempDiffVOF = float(input('      Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
+                while self.MinTempDiffVOF <= 0:
+                    print('          The minimum temperature difference number is not correct. It must be a number larger than 0 and smaller than the maximum temperature difference number. Please enter the number again.')
+                    self.MinTempDiffVOF = float(input('         Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
+
+            while self.MinTempDiffVOF >= self.MaxTempDiffVOF:
+                print('The minimum temperature difference number you entered is larger than or equal to the maximum temperature difference number. Please enter both maximum and minimum temperature difference numbers again.')
+                self.MaxTempDiffVOF = float(input('Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
+                while self.MaxTempDiffVOF <= 0:
+                    print('          The maximum temperature difference number is not correct. It must be a number larger than 0. Please enter the number again.')
+                    self.MaxTempDiffVOF = float(input('         Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
+                while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
+                    self.MaxTempDiffVOF = float(input('      Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
+                    while self.MaxTempDiffVOF <= 0:
+                        print('          The maximum temperature difference number is not correct. It must be a number larger than 0. Please enter the number again.')
+                        self.MaxTempDiffVOF = float(input('         Enter the maximum temperature difference number for Ventilation Opening Factor (any number larger than 0): '))
+
+                self.MinTempDiffVOF = float(input('Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
+                while self.MinTempDiffVOF <= 0:
+                    print('          The minimum temperature difference number is not correct. It must be a number larger than 0 and smaller than the maximum temperature difference number. Please enter the number again.')
+                    self.MinTempDiffVOF = float(input('         Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
+                while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
+                    self.MinTempDiffVOF = float(input('      Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
+                    while self.MinTempDiffVOF <= 0:
+                        print('          The minimum temperature difference number is not correct. It must be a number larger than 0 and smaller than the maximum temperature difference number. Please enter the number again.')
+                        self.MinTempDiffVOF = float(input('         Enter the minimum temperature difference number for Ventilation Opening Factor (any number larger than 0 and smaller than the maximum temperature difference number): '))
+
+            self.MultiplierVOF = float(input('Enter the multiplier number for Ventilation Opening Factor (any number larger than 0): '))
+            while self.MultiplierVOF < 0 or self.MultiplierVOF > 1:
+                print('          The multiplier number is not correct. It must be a number between 0 and 1. Please enter the number again.')
+                self.MultiplierVOF = float(input('         Enter the multiplier number for modulating the Ventilation Opening Factor (any number between 0 and 1): '))
+            while input('          Are you sure the number is correct? [y or [] / n]: ') == 'n':
+                self.MultiplierVOF = float(input('      Enter the multiplier number for modulating the Ventilation Opening Factor (any number between 0 and 1): '))
+                while self.MultiplierVOF < 0 or self.MultiplierVOF > 1:
+                    print('          The multiplier number is not correct. It must be a number between 0 and 1. Please enter the number again.')
+                    self.MultiplierVOF = float(input('         Enter the multiplier number for modulating the Ventilation Opening Factor (any number between 0 and 1): '))
+
         self.VSToffset_List = list(float(num) for num in input(
             "Enter the VSToffset numbers separated by space (if omitted, will be 0): ").split())
         if len(self.VSToffset_List) == 0:
@@ -476,6 +534,9 @@ def inputData(self, ScriptType: str = None):
     elif 'ac' in ScriptType.lower():
         self.HVACmode_List = [0]
         self.VentCtrl_List = [0]
+        self.MaxTempDiffVOF = 1
+        self.MinTempDiffVOF = 0
+        self.MultiplierVOF = 0
         self.VSToffset_List = [0]
         self.MinOToffset_List = [0]
         self.MaxWindSpeed_List = [0]
@@ -519,6 +580,9 @@ def genIDF(self,
            ComfMod=None,
            HVACmode=None,
            VentCtrl=None,
+           MaxTempDiffVOF=20,
+           MinTempDiffVOF=0.5,
+           MultiplierVOF=0.25,
            VSToffset=[0],
            MinOToffset=[50],
            MaxWindSpeed=[50],
@@ -543,6 +607,9 @@ def genIDF(self,
                  ComfMod is None,
                  HVACmode is None,
                  VentCtrl is None,
+                 MaxTempDiffVOF == 20,
+                 MinTempDiffVOF == 0.5,
+                 MultiplierVOF == 0.25,
                  VSToffset == [0],
                  MinOToffset == [50],
                  MaxWindSpeed == [50],
@@ -564,6 +631,9 @@ def genIDF(self,
         self.ComfMod_List = ComfMod
         self.HVACmode_List = HVACmode
         self.VentCtrl_List = VentCtrl
+        self.MaxTempDiffVOF = MaxTempDiffVOF,
+        self.MinTempDiffVOF = MinTempDiffVOF,
+        self.MultiplierVOF = MultiplierVOF,
         self.VSToffset_List = VSToffset
         self.MinOToffset_List = MinOToffset
         self.MaxWindSpeed_List = MaxWindSpeed
@@ -574,10 +644,26 @@ def genIDF(self,
     if 'ac' in ScriptType.lower():
         self.HVACmode_List = [0]
         self.VentCtrl_List = [0]
+        self.MaxTempDiffVOF = 1,
+        self.MinTempDiffVOF = 0,
+        self.MultiplierVOF = 0,
         self.VSToffset_List = [0]
         self.MinOToffset_List = [0]
         self.MaxWindSpeed_List = [0]
 
+    if type(self.MaxTempDiffVOF) is tuple:
+        self.MaxTempDiffVOF = self.MaxTempDiffVOF[0]
+
+    if type(self.MinTempDiffVOF) is tuple:
+        self.MinTempDiffVOF = self.MinTempDiffVOF[0]
+
+    if type(self.MultiplierVOF) is tuple:
+        self.MultiplierVOF = self.MultiplierVOF[0]
+
+    self.VSToffset_List = [float(i) for i in self.VSToffset_List]
+    self.MinOToffset_List = [float(i) for i in self.MinOToffset_List]
+    self.MaxWindSpeed_List = [float(i) for i in self.MaxWindSpeed_List]
+
     if NameSuffix == '':
         suffix = '[NS_X'
     else:
@@ -633,34 +719,38 @@ def genIDF(self,
                                         + '[VO_X'
                                         + '[MT_X'
                                         + '[MW_X'
-                                        + '[AT_' + repr(ASTtol_value)
+                                        + '[AT_' + repr(round(ASTtol_value, 2))
                                         + suffix
                                         + '.idf'
                                 )
                                 outputlist.append(outputname)
                         else:
                             for VentCtrl_value in self.VentCtrl_List:
-                                for VSToffset_value in self.VSToffset_List:
-                                    for MinOToffset_value in self.MinOToffset_List:
-                                        for MaxWindSpeed_value in self.MaxWindSpeed_List:
-                                            for ASTtol_value in numpy.arange(self.ASTtol_value_from,
-                                                                             self.ASTtol_value_to,
-                                                                             self.ASTtol_value_steps):
-                                                outputname = (
-                                                        filename
-                                                        + ComfStand_dict[ComfStand_value]
-                                                        + '[CA_X'
-                                                        + '[CM_X'
-                                                        + '[HM_' + repr(HVACmode_value)
-                                                        + '[VC_' + repr(VentCtrl_value)
-                                                        + '[VO_' + repr(VSToffset_value)
-                                                        + '[MT_' + repr(MinOToffset_value)
-                                                        + '[MW_' + repr(MaxWindSpeed_value)
-                                                        + '[AT_' + repr(ASTtol_value)
-                                                        + suffix
-                                                        + '.idf'
-                                                )
-                                                outputlist.append(outputname)
+                                if HVACmode_value == 1:
+                                    if VentCtrl_value == 2 or VentCtrl_value == 3:
+                                        continue
+                                else:
+                                    for VSToffset_value in self.VSToffset_List:
+                                        for MinOToffset_value in self.MinOToffset_List:
+                                            for MaxWindSpeed_value in self.MaxWindSpeed_List:
+                                                for ASTtol_value in numpy.arange(self.ASTtol_value_from,
+                                                                                 self.ASTtol_value_to,
+                                                                                 self.ASTtol_value_steps):
+                                                    outputname = (
+                                                            filename
+                                                            + ComfStand_dict[ComfStand_value]
+                                                            + '[CA_X'
+                                                            + '[CM_X'
+                                                            + '[HM_' + repr(HVACmode_value)
+                                                            + '[VC_' + repr(VentCtrl_value)
+                                                            + '[VO_' + repr(VSToffset_value)
+                                                            + '[MT_' + repr(MinOToffset_value)
+                                                            + '[MW_' + repr(MaxWindSpeed_value)
+                                                            + '[AT_' + repr(round(ASTtol_value, 2))
+                                                            + suffix
+                                                            + '.idf'
+                                                    )
+                                                    outputlist.append(outputname)
                 elif ComfStand_value in [1, 4, 5]:
                     for CAT_value in self.CAT_List:
                         if ComfStand_value in [1] and CAT_value not in range(0, 4):
@@ -683,7 +773,7 @@ def genIDF(self,
                                                     + '[VO_X'
                                                     + '[MT_X'
                                                     + '[MW_X'
-                                                    + '[AT_' + repr(ASTtol_value)
+                                                    + '[AT_' + repr(round(ASTtol_value, 2))
                                                     + suffix
                                                     + '.idf'
                                             )
@@ -706,7 +796,7 @@ def genIDF(self,
                                                                     + '[VO_' + repr(VSToffset_value)
                                                                     + '[MT_' + repr(MinOToffset_value)
                                                                     + '[MW_' + repr(MaxWindSpeed_value)
-                                                                    + '[AT_' + repr(ASTtol_value)
+                                                                    + '[AT_' + repr(round(ASTtol_value, 2))
                                                                     + suffix
                                                                     + '.idf'
                                                             )
@@ -734,34 +824,38 @@ def genIDF(self,
                                                     + '[VO_X'
                                                     + '[MT_X'
                                                     + '[MW_X'
-                                                    + '[AT_' + repr(ASTtol_value)
+                                                    + '[AT_' + repr(round(ASTtol_value, 2))
                                                     + suffix
                                                     + '.idf'
                                             )
                                             outputlist.append(outputname)
                                     else:
                                         for VentCtrl_value in self.VentCtrl_List:
-                                            for VSToffset_value in self.VSToffset_List:
-                                                for MinOToffset_value in self.MinOToffset_List:
-                                                    for MaxWindSpeed_value in self.MaxWindSpeed_List:
-                                                        for ASTtol_value in numpy.arange(self.ASTtol_value_from,
-                                                                                         self.ASTtol_value_to,
-                                                                                         self.ASTtol_value_steps):
-                                                            outputname = (
-                                                                    filename
-                                                                    + ComfStand_dict[ComfStand_value]
-                                                                    + '[CA_' + repr(CAT_value)
-                                                                    + '[CM_' + repr(ComfMod_value)
-                                                                    + '[HM_' + repr(HVACmode_value)
-                                                                    + '[VC_' + repr(VentCtrl_value)
-                                                                    + '[VO_' + repr(VSToffset_value)
-                                                                    + '[MT_' + repr(MinOToffset_value)
-                                                                    + '[MW_' + repr(MaxWindSpeed_value)
-                                                                    + '[AT_' + repr(ASTtol_value)
-                                                                    + suffix
-                                                                    + '.idf'
-                                                            )
-                                                            outputlist.append(outputname)
+                                            if HVACmode_value == 1:
+                                                if VentCtrl_value == 2 or VentCtrl_value == 3:
+                                                    continue
+                                            else:
+                                                for VSToffset_value in self.VSToffset_List:
+                                                    for MinOToffset_value in self.MinOToffset_List:
+                                                        for MaxWindSpeed_value in self.MaxWindSpeed_List:
+                                                            for ASTtol_value in numpy.arange(self.ASTtol_value_from,
+                                                                                             self.ASTtol_value_to,
+                                                                                             self.ASTtol_value_steps):
+                                                                outputname = (
+                                                                        filename
+                                                                        + ComfStand_dict[ComfStand_value]
+                                                                        + '[CA_' + repr(CAT_value)
+                                                                        + '[CM_' + repr(ComfMod_value)
+                                                                        + '[HM_' + repr(HVACmode_value)
+                                                                        + '[VC_' + repr(VentCtrl_value)
+                                                                        + '[VO_' + repr(VSToffset_value)
+                                                                        + '[MT_' + repr(MinOToffset_value)
+                                                                        + '[MW_' + repr(MaxWindSpeed_value)
+                                                                        + '[AT_' + repr(round(ASTtol_value, 2))
+                                                                        + suffix
+                                                                        + '.idf'
+                                                                )
+                                                                outputlist.append(outputname)
         elif TempCtrl.lower() == 'pmv':
             outputname = (
                     filename
@@ -808,6 +902,8 @@ def genIDF(self,
             # print(filename)
             SetInputData = ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if
                              program.Name == 'SetInputData'])
+            SetVOFinputData = ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if
+                             program.Name == 'SetVOFinputData'])
             if TempCtrl.lower() == 'temp' or TempCtrl.lower() == 'temperature':
                 for ComfStand_value in self.ComfStand_List:
                     SetInputData[0].Program_Line_1 = 'set ComfStand = ' + repr(ComfStand_value)
@@ -831,7 +927,7 @@ def genIDF(self,
                                             + '[VO_X'
                                             + '[MT_X'
                                             + '[MW_X'
-                                            + '[AT_' + repr(ASTtol_value)
+                                            + '[AT_' + repr(round(ASTtol_value, 2))
                                             + suffix
                                             + '.idf'
                                     )
@@ -842,7 +938,14 @@ def genIDF(self,
                                     idf1.savecopy(outputname)
                             else:
                                 for VentCtrl_value in self.VentCtrl_List:
-                                    SetInputData[0].Program_Line_5 = 'set VentCtrl = ' + repr(VentCtrl_value)
+                                    if HVACmode_value == 1:
+                                        if VentCtrl_value == 2 or VentCtrl_value == 3:
+                                            continue
+                                    else:
+                                        SetInputData[0].Program_Line_5 = 'set VentCtrl = ' + repr(VentCtrl_value)
+                                        SetVOFinputData[0].Program_Line_1 = 'set MaxTempDiffVOF = ' + repr(self.MaxTempDiffVOF)
+                                        SetVOFinputData[0].Program_Line_2 = 'set MinTempDiffVOF = ' + repr(self.MinTempDiffVOF)
+                                        SetVOFinputData[0].Program_Line_3 = 'set MultiplierVOF = ' + repr(self.MultiplierVOF)
                                     for VSToffset_value in self.VSToffset_List:
                                         SetInputData[0].Program_Line_6 = 'set VSToffset = ' + repr(VSToffset_value)
                                         for MinOToffset_value in self.MinOToffset_List:
@@ -868,7 +971,7 @@ def genIDF(self,
                                                             + '[VO_' + repr(VSToffset_value)
                                                             + '[MT_' + repr(MinOToffset_value)
                                                             + '[MW_' + repr(MaxWindSpeed_value)
-                                                            + '[AT_' + repr(ASTtol_value)
+                                                            + '[AT_' + repr(round(ASTtol_value, 2))
                                                             + suffix
                                                             + '.idf'
                                                     )
@@ -905,7 +1008,7 @@ def genIDF(self,
                                                         + '[VO_X'
                                                         + '[MT_X'
                                                         + '[MW_X'
-                                                        + '[AT_' + repr(ASTtol_value)
+                                                        + '[AT_' + repr(round(ASTtol_value, 2))
                                                         + suffix
                                                         + '.idf'
                                                 )
@@ -916,8 +1019,14 @@ def genIDF(self,
                                                 idf1.savecopy(outputname)
                                         else:
                                             for VentCtrl_value in self.VentCtrl_List:
-                                                SetInputData[0].Program_Line_5 = 'set VentCtrl = ' + repr(
-                                                    VentCtrl_value)
+                                                if HVACmode_value == 1:
+                                                    if VentCtrl_value == 2 or VentCtrl_value == 3:
+                                                        continue
+                                                else:
+                                                    SetInputData[0].Program_Line_5 = 'set VentCtrl = ' + repr(VentCtrl_value)
+                                                    SetVOFinputData[0].Program_Line_1 = 'set MaxTempDiffVOF = ' + repr(self.MaxTempDiffVOF)
+                                                    SetVOFinputData[0].Program_Line_2 = 'set MinTempDiffVOF = ' + repr(self.MinTempDiffVOF)
+                                                    SetVOFinputData[0].Program_Line_3 = 'set MultiplierVOF = ' + repr(self.MultiplierVOF)
                                                 for VSToffset_value in self.VSToffset_List:
                                                     SetInputData[0].Program_Line_6 = 'set VSToffset = ' + repr(
                                                         VSToffset_value)
@@ -947,7 +1056,7 @@ def genIDF(self,
                                                                         + '[VO_' + repr(VSToffset_value)
                                                                         + '[MT_' + repr(MinOToffset_value)
                                                                         + '[MW_' + repr(MaxWindSpeed_value)
-                                                                        + '[AT_' + repr(ASTtol_value)
+                                                                        + '[AT_' + repr(round(ASTtol_value, 2))
                                                                         + suffix
                                                                         + '.idf'
                                                                 )
@@ -985,7 +1094,7 @@ def genIDF(self,
                                                         + '[VO_X'
                                                         + '[MT_X'
                                                         + '[MW_X'
-                                                        + '[AT_' + repr(ASTtol_value)
+                                                        + '[AT_' + repr(round(ASTtol_value, 2))
                                                         + suffix
                                                         + '.idf'
                                                 )
@@ -996,8 +1105,14 @@ def genIDF(self,
                                                 idf1.savecopy(outputname)
                                         else:
                                             for VentCtrl_value in self.VentCtrl_List:
-                                                SetInputData[0].Program_Line_5 = 'set VentCtrl = ' + repr(
-                                                    VentCtrl_value)
+                                                if HVACmode_value == 1:
+                                                    if VentCtrl_value == 2 or VentCtrl_value == 3:
+                                                        continue
+                                                else:
+                                                    SetInputData[0].Program_Line_5 = 'set VentCtrl = ' + repr(VentCtrl_value)
+                                                    SetVOFinputData[0].Program_Line_1 = 'set MaxTempDiffVOF = ' + repr(self.MaxTempDiffVOF)
+                                                    SetVOFinputData[0].Program_Line_2 = 'set MinTempDiffVOF = ' + repr(self.MinTempDiffVOF)
+                                                    SetVOFinputData[0].Program_Line_3 = 'set MultiplierVOF = ' + repr(self.MultiplierVOF)
                                                 for VSToffset_value in self.VSToffset_List:
                                                     SetInputData[0].Program_Line_6 = 'set VSToffset = ' + repr(
                                                         VSToffset_value)
@@ -1027,7 +1142,7 @@ def genIDF(self,
                                                                         + '[VO_' + repr(VSToffset_value)
                                                                         + '[MT_' + repr(MinOToffset_value)
                                                                         + '[MW_' + repr(MaxWindSpeed_value)
-                                                                        + '[AT_' + repr(ASTtol_value)
+                                                                        + '[AT_' + repr(round(ASTtol_value, 2))
                                                                         + suffix
                                                                         + '.idf'
                                                                 )
