@@ -8,6 +8,7 @@ by adding the Adaptive Comfort Control Implementation Script (ACCIS)
 
 def addAccis(
         ScriptType: str = None,
+        SupplyAirTempInputMethod: str = None,
         Output_type: str = None,
         Output_freqs: any = None,
         Output_keep_existing: bool = None,
@@ -18,6 +19,9 @@ def addAccis(
         ComfMod: any = None,
         HVACmode: any = None,
         VentCtrl: any = None,
+        MaxTempDiffVOF: float = 20,
+        MinTempDiffVOF: float = 0.5,
+        MultiplierVOF: float = 0.25,
         VSToffset: any = [0],
         MinOToffset: any = [50],
         MaxWindSpeed: any = [50],
@@ -26,7 +30,8 @@ def addAccis(
         ASTtol_steps: float = 0.1,
         NameSuffix: str = '',
         verboseMode: bool = True,
-        confirmGen: bool = None):
+        confirmGen: bool = None
+):
     """
     Parameters
     :param ScriptType: The default is None.
@@ -34,7 +39,9 @@ def addAccis(
     'vrf_mm' for VRFsystem with mixed-mode, 
     'ex_ac' for ExistingHVAC only with full air-conditioning mode, 
     'ex_mm' for ExistingHVAC with mixed-mode.
-    :param Output_type: The default is None. Can be 'standard', 'simplified' or 'timestep'.
+    :param Output_type: The default is None. Can be 'standard', 'simplified' or 'detailed'.
+    :param Output_freqs: The default is None. A list containing the following strings: ['timestep', 'hourly', 'daily', 'monthly', 'runperiod']
+    :param Output_keep_existing: The default is None. It is a boolean (True or False) to keep the existing Output:Variable objects or not.
     :param EnergyPlus_version: The default is None. Can be '9.1', '9.2', '9.3', '9.4', '9.5', '9.6', '22.1' or '22.2'.
     :param TempCtrl: The default is None. Can be 'temp' or 'pmv'.
     :param ComfStand: The default is None.
@@ -59,6 +66,7 @@ def addAccis(
         '18 = MEX Oropeza DryTropic;
         '19 = MEX Oropeza Temperate;
         '20 = MEX Oropeza HumTropic;
+        '21 = CHL Perez-Fargallo;
     :param CAT: The default is None.
     (1 = CAT I; 2 = CAT II; 3 = CAT III; 80 = 80% ACCEPT; 85 = 85% ACCEPT; 90 = 90% ACCEPT)
     :param ComfMod: The default is None.
@@ -69,7 +77,18 @@ def addAccis(
     :param HVACmode: The default is None.
     (0 = Fully Air-conditioned; 1 = Naturally ventilated; 2 = Mixed Mode)
     :param VentCtrl: The default is None.
-    (0 = Ventilates above neutral temperature; 1 = Ventilates above upper comfort limit)
+    (if HVACmode = 1:
+    0 = Ventilates above neutral temperature;
+    1 = Ventilates above upper comfort limit;
+    if HVACmode = 2:
+    0 = Ventilates above neutral temperature and fully opens doors and windows;
+    1 = Ventilates above lower comfort limit and fully opens doors and windows;
+    2 = Ventilates above neutral temperature and opens doors and windows based on the customised venting opening factor;
+    3 = Ventilates above lower comfort limit and opens doors and windows based on the customised venting opening factor;
+    )
+    :param MaxTempDiffVOF: The maximum temperature difference for the Venting Opening Factor. Must be a number greater than 0.
+    :param MinTempDiffVOF: The minimum temperature difference for the Venting Opening Factor. Must be a number greater than 0 and smaller than the maximum temperature difference.
+    :param MultiplierVOF: The multiplier for the modulation of the Venting Opening Factor. Must be a number between 0 and 1.
     :param VSToffset: The default is 0. Please refer to documentation.
     :param MinOToffset: The default is 50. Please refer to documentation.
     :param MaxWindSpeed: The default is 50. Please refer to documentation.
@@ -93,12 +112,14 @@ def addAccis(
     import accim
 
     filelist = ([file for file in listdir() if file.endswith('.idf')
-                 and not '[' in file])
+                 and not '[' in file
+                 and not '_pymod' in file])
 
     filelist = ([file.split('.idf')[0] for file in filelist])
 
     objArgsDef = (
         ScriptType is not None,
+        SupplyAirTempInputMethod is not None,
         Output_type is not None,
         Output_freqs is not None,
         Output_keep_existing is not None,
@@ -112,11 +133,18 @@ def addAccis(
                           'ex_ac',
                           ]
 
+    SupplyAirTempInputMethodList = [
+        'supply air temperature',
+        'temperature difference'
+    ]
+
     fullOutputsTypeList = [
         'Standard',
         'standard',
         'Simplified',
         'simplified',
+        'Detailed',
+        'detailed'
     ]
 
     fullOutputsFreqList = [
@@ -166,14 +194,26 @@ def addAccis(
                                "    for ExistingHVAC with mixed mode: ex_mm;\n"
                                "    for ExistingHVAC with full air-conditioning mode: ex_ac\n"
                                "    ): ")
+        if 'vrf' in ScriptType.lower():
+            SupplyAirTempInputMethod = input("Enter the SupplyAirTempInputMethod (\n"
+                               "for Supply Air Temperature: supply air temperature;\n"
+                               "for Temperature Difference: temperature difference;\n"
+                               "): ")
+            while SupplyAirTempInputMethod not in SupplyAirTempInputMethodList:
+                SupplyAirTempInputMethod = input(
+                    "    SupplyAirTempInputMethod was not correct. "
+                    "    Enter the SupplyAirTempInputMethod (\n"
+                               "for Supply Air Temperature: supply air temperature;\n"
+                               "for Temperature Difference: temperature difference;\n"
+                               "): ")
         Output_keep_existing = input('Do you want to keep the existing outputs (true or false)?: ')
         while Output_keep_existing.lower() not in ['true', 'false']:
             Output_keep_existing = input('The answer you entered is not valid. '
                                           'Do you want to keep the existing outputs (true or false)?: ')
-        Output_type = input("Enter the Output type (standard or simplified): ")
+        Output_type = input("Enter the Output type (standard, simplified or detailed): ")
         while Output_type not in fullOutputsTypeList:
             Output_type = input("   Output type was not correct. "
-                            "Please, enter the Output type (standard or simplified): ")
+                            "Please, enter the Output type (standard, simplified or detailed): ")
         Output_freqs = list(freq for freq in input(
             "Enter the Output frequencies separated by space (timestep, hourly, daily, monthly, runperiod): ").split())
         while (not(all(elem in fullOutputsFreqList for elem in Output_freqs))):
@@ -190,7 +230,6 @@ def addAccis(
             TempCtrl = input("  Temperature Control method was not correct. "
                              "Please, enter the Temperature Control method (temperature or pmv): ")
 
-
     if verboseMode:
         print('Basic input data:')
         # print(f'accim version: {accim.__version__}')
@@ -200,6 +239,14 @@ def addAccis(
         print(fullScriptTypeList)
         raise ValueError(ScriptType + " is not a valid ScriptType. "
                                       "You must choose a ScriptType from the list above.")
+    if 'vrf' in ScriptType.lower():
+        if verboseMode:
+            print('Supply Air Temperature Input Method is: '+SupplyAirTempInputMethod)
+        if SupplyAirTempInputMethod not in SupplyAirTempInputMethodList:
+            print('Valid Supply Air Temperature Input Methods: ')
+            print(SupplyAirTempInputMethod)
+            raise ValueError(SupplyAirTempInputMethod + " is not a valid Supply Air Temperature Input Method. "
+                                          "You must choose a Supply Air Temperature Input Method from the list above.")
     if verboseMode:
         print('Output type is: ' + Output_type)
     if Output_type not in fullOutputsTypeList:
@@ -263,7 +310,7 @@ def addAccis(
             z.setAvailSchOn(verboseMode=verboseMode)
             z.addVRFsystemSch(verboseMode=verboseMode)
             z.addCurveObj(verboseMode=verboseMode)
-            z.addDetHVACobj(EnergyPlus_version=EnergyPlus_version, verboseMode=verboseMode, TempCtrl=TempCtrl)
+            z.addDetHVACobj(EnergyPlus_version=EnergyPlus_version, verboseMode=verboseMode, SupplyAirTempInputMethod=SupplyAirTempInputMethod)
             if ScriptType.lower() == 'vrf_mm':
                 z.checkVentIsOn(verboseMode=verboseMode)
             z.addForscriptSchVRFsystem(verboseMode=verboseMode)
@@ -306,27 +353,17 @@ def addAccis(
                 TempCtrl=TempCtrl,
                 verboseMode=verboseMode
             )
-
-        # if Output_type.lower() == 'simplified':
-        #     z.addSimplifiedOutputVariables(
-        #         TempCtrl=TempCtrl,
-        #         verboseMode=verboseMode
-        #     )
-        # elif Output_type.lower() == 'standard':
-        #     z.addOutputVariablesBase(
-        #         ScriptType=ScriptType,
-        #         TempCtrl=TempCtrl,
-        #         verboseMode=verboseMode
-        #     )
-        #
-        # if Output_type.lower() == 'timestep':
-        #     z.addOutputVariablesTimestep(verboseMode=verboseMode)
-        #
-        # if Output_type.lower() == 'runperiod':
-        #     z.addOutputVariablesRunperiod(
-        #         ScriptType=ScriptType,
-        #         verboseMode=verboseMode
-        #     )
+        elif Output_type.lower() == 'detailed':
+            z.addOutputVariablesStandard(
+                Outputs_freq=Output_freqs,
+                ScriptType=ScriptType,
+                TempCtrl=TempCtrl,
+                verboseMode=verboseMode
+            )
+            z.addOutputVariablesDetailed(
+                Outputs_freq=Output_freqs,
+                verboseMode=verboseMode
+            )
 
         z.removeDuplicatedOutputVariables()
 
@@ -378,6 +415,9 @@ def addAccis(
                 ComfMod=ComfMod,
                 HVACmode=HVACmode,
                 VentCtrl=VentCtrl,
+                MaxTempDiffVOF=MaxTempDiffVOF,
+                MinTempDiffVOF=MinTempDiffVOF,
+                MultiplierVOF=MultiplierVOF,
                 VSToffset=VSToffset,
                 MinOToffset=MinOToffset,
                 MaxWindSpeed=MaxWindSpeed,
@@ -407,6 +447,9 @@ def addAccis(
                 HVACmode=[0],
                 VentCtrl=[0],
                 VSToffset=[0],
+                MaxTempDiffVOF=1,
+                MinTempDiffVOF=0,
+                MultiplierVOF=0,
                 MinOToffset=[0],
                 MaxWindSpeed=[0],
                 ASTtol_start=ASTtol_start,
